//
//  KeychainProxy.m
//  Keychain
//
//  Created by Jiri Dutkevic on 17/06/15.
//  Copyright (c) 2015 Avast. All rights reserved.
//

#import "KeychainProxy.h"
#import <Security/Security.h>

@implementation KeychainProxy

- (NSData *)getData:(NSString *)itemId
{
    NSMutableDictionary *dict = [self baseKeychainDictionaryWithItem:itemId];
    [dict setObject:(__bridge id)kSecMatchLimitOne
             forKey:(__bridge id)kSecMatchLimit];
    [dict setObject:(__bridge id)kCFBooleanTrue
             forKey:(__bridge id)kSecReturnData];
    
    CFTypeRef result = nil;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)dict,
                                          &result);
    
#if DEBUG
    NSLog(@"Get Data: Item: %@, Status: %d", itemId, (int)status);
#endif
    
    if (status != errSecSuccess) {
        result = nil;
    }
    return (__bridge_transfer NSData *)result;
}

- (BOOL)setData:(NSData *)dt
        forItem:(NSString *)itemId
{
    BOOL hasData = [self isItemPresent:itemId];
    BOOL hasValue = dt != nil;
    
    if (hasData && hasValue) {
        return [self updateData:dt
                        forItem:itemId
          withSecurityAttribute:kSecAttrAccessibleWhenUnlocked];
    } else if (hasData && !hasValue) {
        return [self deleteItem:itemId];
    } else if (!hasData && hasValue) {
        return [self addData:dt
                     forItem:itemId
       withSecurityAttribute:kSecAttrAccessibleWhenUnlocked];
    } else {
        return YES;
    }
}

- (NSString *)getString:(NSString *)itemId
{
    NSData *data = [self getData:itemId];
    if (data == nil) {
        return nil;
    } else {
        return [[NSString alloc] initWithData:data
                                     encoding:NSUTF8StringEncoding];
    }
}

- (BOOL)setString:(NSString *)str
          forItem:(NSString *)itemId
{
    NSData *data = str == nil ? nil : [str
                                       dataUsingEncoding:NSUTF8StringEncoding];
    return [self setData:data
                 forItem:itemId];
}

- (BOOL)isItemPresent:(NSString *)itemId
{
    NSMutableDictionary *dict = [self baseKeychainDictionaryWithItem:itemId];
    CFTypeRef result = nil;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)dict,
                                          &result);
    if (result != nil) {
        CFRelease(result);
    }
#if DEBUG
    NSLog(@"Presence: Item: %@, Status: %d", itemId, (int)status);
#endif
    
    return status == errSecSuccess;
}

- (BOOL)deleteItem:(NSString *)itemId
{
    NSMutableDictionary *dict = [self baseKeychainDictionaryWithItem:itemId];
    OSStatus status = SecItemDelete((__bridge CFDictionaryRef)dict);
    
#if DEBUG
    NSLog(@"Delete: Item: %@, Status: %d", itemId, (int)status);
#endif
    
    return status == errSecSuccess;
}

- (BOOL)addData:(NSData *)dt
        forItem:(NSString *)itemId
withSecurityAttribute:(CFStringRef)secAttr
{
    NSMutableDictionary *dict = [self baseKeychainDictionaryWithItem:itemId];
    [dict setObject:(__bridge id)secAttr
             forKey:(__bridge id)kSecAttrAccessible];
    [dict setObject:dt
             forKey:(__bridge id)kSecValueData];
    
    CFTypeRef result = nil;
    OSStatus status = SecItemAdd((__bridge CFDictionaryRef)dict,
                                 &result);
#if DEBUG
    NSLog(@"Add: Item: %@, Status: %d", itemId, (int)status);
#endif
    
    return status == errSecSuccess;
}

- (BOOL)updateData:(NSData *)dt
           forItem:(NSString *)itemId
withSecurityAttribute:(CFStringRef)secAttr
{
    NSMutableDictionary *searchDict = [self
                                       baseKeychainDictionaryWithItem:itemId];
    NSDictionary *updateDict = @{
                                 (__bridge id)kSecAttrAccessible: (__bridge id)secAttr,
                                 (__bridge id)kSecValueData: dt
                                 };
    
    OSStatus status = SecItemUpdate((__bridge CFDictionaryRef)searchDict,
                                    (__bridge CFDictionaryRef)updateDict);
#if DEBUG
    NSLog(@"Update: Item: %@, Status: %d", itemId, (int)status);
#endif
    
    return status == errSecSuccess;
}

- (BOOL)addString:(NSString *)str
          forItem:(NSString *)itemId
withSecurityAttribute:(CFStringRef)secAttr
{
    return [self addData:[str dataUsingEncoding:NSUTF8StringEncoding]
                 forItem:itemId
   withSecurityAttribute:secAttr];
}

- (BOOL)updateString:(NSString *)str
             forItem:(NSString *)itemId
withSecurityAttribute:(CFStringRef)secAttr
{
    return [self updateData:[str dataUsingEncoding:NSUTF8StringEncoding]
                    forItem:itemId
      withSecurityAttribute:secAttr];
}

- (NSMutableDictionary *)baseKeychainDictionaryWithItem:(NSString *)itemId
{
    NSString *service = @"demo";
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    [dict setObject:(__bridge id)kSecClassGenericPassword
             forKey:(__bridge id)kSecClass];
    [dict setObject:[NSString stringWithFormat:@"%@.%@", service, itemId]
             forKey:(__bridge id)kSecAttrAccount];
    [dict setObject:service forKey:(__bridge id)kSecAttrService];
    return dict;
}

@end
