//
//  Encryptor.m
//  FFL2CommonCrypto
//
//  Created by Jiri Dutkevic on 21/12/15.
//  Copyright © 2015 Avast. All rights reserved.
//

#import "Encryptor.h"
#import <CommonCrypto/CommonCrypto.h>
#import <CommonCrypto/CommonRandom.h>

@interface Encryptor ()

+ (nullable NSData *)aes:(CCOperation)operation data:(nonnull NSData *)data key:(nonnull NSData *)key iv:(nonnull NSData *)iv;

@end

@implementation Encryptor

+ (nullable NSData *)randomBytes:(NSInteger)size
{
    if (size <= 0) {
        return nil;
    }
    NSMutableData *data = [NSMutableData dataWithLength:size];
    CCRNGStatus rngstatus = CCRandomGenerateBytes(data.mutableBytes, (size_t)size);
    if (rngstatus != kCCSuccess) {
        return nil;
    }
    return data;
}

+ (NSInteger)aesBlockSize
{
    return (NSInteger)kCCBlockSizeAES128;
}

+ (NSInteger)aesKeySize
{
    return (NSInteger)kCCKeySizeAES256;
}

+ (NSInteger)sha256Size
{
    return (NSInteger)CC_SHA256_DIGEST_LENGTH;
}

+ (nullable NSData *)aesEncryptData:(nonnull NSData *)data key:(nonnull NSData *)key iv:(nonnull NSData *)iv
{
    return [self aes:kCCEncrypt data:data key:key iv:iv];
}

+ (nullable NSData *)aesDecryptData:(nonnull NSData *)data key:(nonnull NSData *)key iv:(nonnull NSData *)iv
{
    return [self aes:kCCDecrypt data:data key:key iv:iv];
}

+ (nullable NSData *)aes:(CCOperation)operation data:(nonnull NSData *)data key:(nonnull NSData *)key iv:(nonnull NSData *)iv
{
    if (data == nil || data.length == 0) {
        return nil;
    }
    if (key == nil || key.length != self.aesKeySize) {
        return nil;
    }
    if (iv == nil || iv.length != self.aesBlockSize) {
        return nil;
    }
    
    CCCryptorStatus status;
    
    CCCryptorRef cryptor = nil;
    status = CCCryptorCreate(operation, kCCAlgorithmAES, iv == nil ? 0 : kCCOptionPKCS7Padding, key.bytes, (size_t)key.length, iv.bytes, &cryptor);
    if (status != kCCSuccess || cryptor == nil) {
        return nil;
    }
    
    size_t neededSpace = CCCryptorGetOutputLength(cryptor, (size_t)data.length, true);
    NSMutableData *result = [NSMutableData dataWithLength:(NSUInteger)neededSpace];
    
    size_t writtenDataUpdate = 0;
    status = CCCryptorUpdate(cryptor, data.bytes, (size_t)data.length, result.mutableBytes, neededSpace, &writtenDataUpdate);
    if (status != kCCSuccess) {
        CCCryptorRelease(cryptor);
        return nil;
    }
    
    size_t writtenDataFinal = 0;
    status = CCCryptorFinal(cryptor, result.mutableBytes+writtenDataUpdate, neededSpace-writtenDataUpdate, &writtenDataFinal);
    if (status != kCCSuccess) {
        CCCryptorRelease(cryptor);
        return nil;
    }
    
    CCCryptorRelease(cryptor);
    
    return [[NSData alloc] initWithBytes:result.bytes length:(NSUInteger)(writtenDataUpdate+writtenDataFinal)];
}

+ (nullable NSData *)sha256Data:(nonnull NSData *)data
{
    if (data == nil || data.length == 0) {
        return nil;
    }
    NSMutableData *sha = [NSMutableData dataWithLength:(NSUInteger)self.sha256Size];
    CC_SHA256(data.bytes, (CC_LONG)data.length, sha.mutableBytes);
    return sha;
}

+ (nullable NSData *)deriveKey:(nonnull NSData *)key
{
    NSMutableData *data = [[NSMutableData alloc] initWithLength:self.aesKeySize];
    CCKeyDerivationPBKDF(kCCPBKDF2, key.bytes, key.length,
                         NULL, 0,
                         kCCPRFHmacAlgSHA256, 100,
                         data.mutableBytes, self.aesKeySize);
    return data;
}

@end
